﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using Microsoft.Extensions.Logging;
using MotorPlantContracts.VeiwModels;

namespace MotorPlantBusinessLogic.BusinessLogic
{
    public class EngineLogic : IEngineLogic
    {
        private readonly ILogger _logger;

        private readonly IEngineStorage _engineStorage;

        public EngineLogic(ILogger<EngineLogic> logger, IEngineStorage engineStorage)
        {
            _logger = logger;
            _engineStorage = engineStorage;
        }
        public bool Create(EngineBindingModel model)
        {
            CheckModel(model);
            if (_engineStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(EngineBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_engineStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        public EngineViewModel? ReadElement(EngineSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. EngineName:{EngineName}.Id:{ Id}", model.EngineName, model.Id);
            var element = _engineStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public List<EngineViewModel>? ReadList(EngineSearchModel? model)
        {
            _logger.LogInformation("ReadList. EngineName:{EngineName}.Id:{ Id}", model?.EngineName, model?.Id);
            var list = model == null ? _engineStorage.GetFullList() : _engineStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public bool Update(EngineBindingModel model)
        {
            CheckModel(model);
            if (_engineStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(EngineBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.EngineName))
            {
                throw new ArgumentNullException("Нет названия изделия", nameof(model.EngineName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена изделия должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Engine. EngineName:{EngineName}.Price:{ Cost}. Id: { Id}", model.EngineName, model.Price, model.Id);
            var element = _engineStorage.GetElement(new EngineSearchModel
            {
                EngineName = model.EngineName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Изделие с таким названием уже есть");
            }
        }
    }
}
